--------------------------------------------------------------------------------
-- Beispielskript zur AEK 20 im Oktober 2017 in Nrnberg und Hannover
-- Autor: Bernd Jungbluth
-- Keine Gewhrleistung, keine Haftung

-- Auswerten einer XEVENT-Sitzung zur Protokollierung von Sperren und Wartetypen
--------------------------------------------------------------------------------

USE AEK20;
GO

-- Zieltabelle lschen, falls bereits vorhanden
DROP TABLE IF EXISTS XEventResults;
GO

-- Zieltabelle anlegen
CREATE TABLE XEventResults 
(
	EventSequence		BIGINT			NOT NULL,
	EventName			SYSNAME			NOT NULL,
	SessionID			INT				NOT NULL,
	LockMode			SYSNAME			NULL,
	SqlText				NVARCHAR(MAX)	NULL,
	WaitInfo			SYSNAME			NULL,
	ResourceDescription	SYSNAME			NULL,
	BlockedSessionID	INT				NULL,
	BlockingSessionID	INT				NULL,
	BlockedSqlText		NVARCHAR(MAX)	NULL,
	BlockingSqlText		NVARCHAR(MAX)	NULL,
	BlockedUser			SYSNAME			NULL,
	BlockingUser		SYSNAME			NULL,
	BlockedClient		SYSNAME			NULL,
	BlockingClient		SYSNAME			NULL,
	BlockedApp			SYSNAME			NULL,
	BlockingApp			SYSNAME			NULL,
	WaitResource		SYSNAME			NULL,
	DatabaseID			INT				NULL,
	DatabaseName		SYSNAME			NULL,
	Username			SYSNAME			NULL,
	ClientHostname		SYSNAME			NULL,
	ClientAppName		SYSNAME			NULL,
	TimeStampNotUTC		DATETIME2		NOT NULL,
	Ergebnis			XML				NOT NULL
);
GO

-- Daten in Zieltabelle schreiben
INSERT INTO XEventResults
SELECT
	Ergebnis.value('(/event/action[@name=''event_sequence'']/value)[1]','BIGINT') AS EventSequence,
	Ereignis As EventName, 
	Ergebnis.value('(/event/action[@name=''session_id'']/value)[1]','INT') AS SessionID,
	COALESCE(Ergebnis.value('(/event/data[@name=''mode'']/text)[1]','SYSNAME'), 
		 	 Ergebnis.value('(/event/data[@name=''lock_mode'']/text)[1]','SYSNAME')) AS LockMode,
	Ergebnis.value('(/event/action[@name=''sql_text'']/value)[1]','NVARCHAR(MAX)') AS SqlText,
	Ergebnis.value('(/event/data[@name=''wait_type'']/text)[1]','SYSNAME') AS WaitInfo,
	Ergebnis.value('(/event/data[@name=''resource_description'']/value)[1]','SYSNAME') AS ResourceDescription,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocked-process/process/@spid)[1]','SYSNAME') AS BlockedSessionID,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocking-process/process/@spid)[1]','SYSNAME') AS BlockingSessionID,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocked-process/process/inputbuf)[1]','NVARCHAR(MAX)') AS BlockedSqlText,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocking-process/process/inputbuf)[1]','NVARCHAR(MAX)') AS BlockingSqlText,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocked-process/process/@loginname)[1]','SYSNAME') AS BlockedUser,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocking-process/process/@loginname)[1]','SYSNAME') AS BlockingUser,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocked-process/process/@hostname)[1]','SYSNAME') AS BlockedClient,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocking-process/process/@hostname)[1]','SYSNAME') AS BlockingClient,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocked-process/process/@clientapp)[1]','SYSNAME') AS BlockedApp,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocking-process/process/@clientapp)[1]','SYSNAME') AS BlockingApp,
	Ergebnis.value('(/event/data[@name=''blocked_process'']/value/blocked-process-report/blocked-process/process/@waitresource)[1]','SYSNAME') AS WaitResource, 
	CASE WHEN Ereignis = 'blocked_process_report' 
		 THEN Ergebnis.value('(/event/data[@name=''database_id'']/value)[1]','INT')
		 ELSE Ergebnis.value('(/event/action[@name=''database_id'']/value)[1]','INT') END AS DatabaseID,
	CASE WHEN Ereignis = 'blocked_process_report' 
		 THEN Ergebnis.value('(/event/data[@name=''database_name'']/value)[1]','SYSNAME')
		 ELSE Ergebnis.value('(/event/action[@name=''database_name'']/value)[1]','SYSNAME') END AS DatabaseName,
	CASE WHEN Ereignis = 'blocked_process_report' THEN NULL ELSE Ergebnis.value('(/event/action[@name=''username'']/value)[1]','SYSNAME') END AS Username,
	CASE WHEN Ereignis = 'blocked_process_report' THEN NULL ELSE Ergebnis.value('(/event/action[@name=''client_hostname'']/value)[1]','SYSNAME') END AS ClientHostname,
	CASE WHEN Ereignis = 'blocked_process_report' THEN NULL ELSE Ergebnis.value('(/event/action[@name=''client_app_name'']/value)[1]','SYSNAME') END AS ClientAppName,
	DATEADD(hh, DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
					Ergebnis.value('(event/@timestamp)[1]', 'datetime2')) AS TimeStampNotUTC,
	Ergebnis
FROM
	(SELECT OBJECT_NAME AS Ereignis, CONVERT(XML, event_data) AS Ergebnis
	 FROM	sys.fn_xe_file_target_read_file('D:\AEK 20\SQL Server\XEvents\Sperren und Wartetypen*.xel',NULL,NULL,NULL)) AS Ergebnisdaten
ORDER BY 
	Ergebnis.value('(/event/action[@name=''event_sequence'']/value)[1]','BIGINT');


-- Inhalt der Zieltabelle ausgeben
SELECT * FROM XEventResults ORDER BY EventSequence;




-- Zieltabelle auswerten
WITH cteWaits (SessionID, WaitInfo)
AS
(
	SELECT	DISTINCT SessionID, WaitInfo
	FROM	XEventResults 
	WHERE	WaitInfo IS NOT NULL
) 
SELECT	DISTINCT
		c.SessionID, c.WaitInfo, 
		x.WaitResource, x.BlockedSessionID, x.BlockingSessionID,
		CASE WHEN c.SessionID = x.BlockedSessionID
			 THEN BlockedSqlText ELSE BlockingSqlText END As SqlText,
		x.DatabaseID, x.DatabaseName, 
		CASE WHEN c.SessionID = x.BlockedSessionID
			 THEN BlockedUser ELSE BlockingUser END As Username,
		CASE WHEN c.SessionID = x.BlockedSessionID
			 THEN BlockedClient ELSE BlockingClient END As ClientHostName,
		CASE WHEN c.SessionID = x.BlockedSessionID
			 THEN BlockedApp ELSE BlockingApp END As ClientAppName
FROM	
		cteWaits c
		INNER JOIN	XEventResults x 
					ON	c.SessionID = x.BlockedSessionID 
						OR 
						c.SessionID = x.BlockingSessionID;		